import { loadPolicy } from "@open-policy-agent/opa-wasm";


export async function createPolicyCheck(user?: Partial<Oidc.Profile>) {
    if (!user) {
        return () => true;
    }

    const opaBundletUrl = process.env.OPA_BUNDLE_URL;

    if (typeof opaBundletUrl === 'undefined') {
        console.log('OPA_BUNDLE_URL is not defined');
        return () => true;
    }

    const policyResult = await fetch(opaBundletUrl);
    const policyWasm = await policyResult.arrayBuffer();
    try {
        const policy = await loadPolicy(policyWasm);

        function allowed(resource: string): boolean {
            const input: any = {
                resource: resource,
                method: "GET",
                ...user,
            };

            const resultSet = policy.evaluate(input);

            if (resultSet == null || resultSet.length === 0) {
                console.error("evaluation error", resultSet);
                return false;
            }

            return resultSet[0].result;
        }
        return allowed;
    } catch {
        console.error("policy evaluation error");
        return (_: string) => false;
    }
}
